<?php
require_once '../database_config.php';

// 检查用户是否有权限访问此页面
check_permission_or_die('checkin');

// 记录访问日志
record_log('访问入住与退房管理首页');

// 获取统计数据
try {
    // 查询今日入住人数
    $today = date('Y-m-d');
    $stmt = $pdo->prepare("SELECT COUNT(*) FROM checkin_orders WHERE DATE(checkin_time) = ?");
    $stmt->execute([$today]);
    $checkin_count = $stmt->fetchColumn();

    // 查询今日退房人数
    $stmt = $pdo->prepare("SELECT COUNT(*) FROM checkin_orders WHERE DATE(checkout_time) = ?");
    $stmt->execute([$today]);
    $checkout_count = $stmt->fetchColumn();

    // 查询当前在住人数
    $stmt = $pdo->query("SELECT COUNT(*) FROM checkin_orders WHERE status = 1");
    $staying_count = $stmt->fetchColumn();

    // 查询房间总数和可用房间数
    $stmt = $pdo->query("SELECT COUNT(*) FROM room_info");
    $total_rooms = $stmt->fetchColumn();

    $stmt = $pdo->query("SELECT COUNT(*) FROM room_info WHERE status = 1 AND id NOT IN (SELECT room_id FROM checkin_orders WHERE status = 1)");
    $available_rooms = $stmt->fetchColumn();
} catch (PDOException $e) {
    error_log("数据库查询错误: " . $e->getMessage());
    $checkin_count = $checkout_count = $staying_count = $total_rooms = $available_rooms = 0;
}

// 获取最近入住记录
try {
    $stmt = $pdo->query("
        SELECT c.*, r.room_number 
        FROM checkin_orders c
        LEFT JOIN room_info r ON c.room_id = r.id
        ORDER BY c.checkin_time DESC
        LIMIT 5
    ");
    $recent_checkins = $stmt->fetchAll();
} catch (PDOException $e) {
    error_log("数据库查询错误: " . $e->getMessage());
    $recent_checkins = [];
}
?>

<!DOCTYPE html>
<html lang="zh-CN">

<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title><?php echo title; ?></title>
    <link rel="stylesheet" href="../style.css">
</head>

<body>
    <div class="container">
        <div class="header-local">
            <h2>入住与退房管理</h2>
        </div>

        <div class="content">
            <!-- 统计数据卡片 -->
            <div class="dashboard">
                <div class="dashboard-card">
                    <h3>当前在住</h3>
                    <div class="number"><?php echo $staying_count; ?></div>
                    <div class="desc">入住状态客人数量</div>
                </div>

                <div class="dashboard-card">
                    <h3>空闲房间</h3>
                    <div class="number"><?php echo $available_rooms; ?></div>
                    <div class="desc">可用房间总数量</div>
                </div>

                <div class="dashboard-card">
                    <h3>今日入住</h3>
                    <div class="number"><?php echo $checkin_count; ?></div>
                    <div class="desc">今日办理入住数量</div>
                </div>

                <div class="dashboard-card">
                    <h3>今日退房</h3>
                    <div class="number"><?php echo $checkout_count; ?></div>
                    <div class="desc">今日办理退房数量</div>
                </div>
            </div>

            <!-- 功能导航 -->
            <h2>功能导航</h2>
            <div class="function-links">
                <a href="checkin.php" class="function-link">
                    <h3>入住管理</h3>
                </a>
                <a href="checkout.php" class="function-link">
                    <h3>退房结算</h3>
                </a>
                <a href="deposit.php" class="function-link">
                    <h3>押金管理</h3>
                </a>
                <a href="invoice.php" class="function-link">
                    <h3>电子发票</h3>
                </a>
            </div>

            <!-- 最近入住记录 -->
            <h2>最近入住记录</h2>
            <div class="system-info">
                <table class="info-table">
                    <tr>
                        <td>入住单号</td>
                        <td>客人姓名</td>
                        <td>房间号</td>
                        <td>入住时间</td>
                        <td>状态</td>
                    </tr>
                    <?php foreach ($recent_checkins as $checkin):
                        $status_text = '';
                        if ($checkin['status'] == 1) {
                            $status_text = '在住';
                        } elseif ($checkin['status'] == 2) {
                            $status_text = '已退房';
                        } elseif ($checkin['status'] == 3) {
                            $status_text = '已取消';
                        }
                    ?>
                        <tr>
                            <td><?php echo $checkin['checkin_no']; ?></td>
                            <td><?php echo $checkin['guest_name']; ?></td>
                            <td><?php echo $checkin['room_number']; ?></td>
                            <td><?php echo date('Y-m-d H:i', strtotime($checkin['checkin_time'])); ?></td>
                            <td><?php echo $status_text; ?></td>
                        </tr>
                    <?php endforeach; ?>

                    <?php if (empty($recent_checkins)): ?>
                        <tr>
                            <td colspan="5" class="text-center">暂无入住记录</td>
                        </tr>
                    <?php endif; ?>
                </table>
            </div>
        </div>
    </div>
</body>

</html>