<?php
session_start();

define('DB_HOST', 'localhost'); //数据库主机地址
define('DB_NAME', 'hotelpro'); //数据库名称
define('DB_USER', 'root'); //数据库用户名
define('DB_PASS', 'root'); //数据库密码
define('title', '智慧酒店管理系统（HotelPro）'); //系统名称

try {
    $pdo = new PDO(
        "mysql:host=" . DB_HOST . ";dbname=" . DB_NAME . ";charset=utf8mb4",
        DB_USER,
        DB_PASS,
        [
            PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
            PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC
        ]
    );
} catch (PDOException $e) {
    die("数据库连接失败: " . $e->getMessage());
}

// 以下内容需要按照实际数据库表的结构进行修改
/**
 * 记录操作日志

 * @param string $operation 操作描述
 * @return bool 是否记录成功
 */
function record_log($operation)
{
    global $pdo;

    $user_id = isset($_SESSION['user_id']) ? $_SESSION['user_id'] : null;
    $username = isset($_SESSION['username']) ? $_SESSION['username'] : null;
    $ip = get_client_ip(); // 使用已有的获取IP地址函数

    try {
        $stmt = $pdo->prepare("INSERT INTO sys_logs (user_id, username, operation, ip) VALUES (?, ?, ?, ?)");
        return $stmt->execute([$user_id, $username, $operation, $ip]);
    } catch (PDOException $e) {
        // 日志记录失败，但不影响主程序运行
        error_log("记录日志失败: " . $e->getMessage());
        return false;
    }
}

/**
 * 获取客户端IP地址
 * @return string 客户端IP地址
 */
function get_client_ip()
{
    if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
        $ip = $_SERVER['HTTP_CLIENT_IP'];
    } elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
        $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
    } else {
        $ip = $_SERVER['REMOTE_ADDR'];
    }
    return $ip;
}

/**
 * 检查当前用户是否有指定权限
 * @param string $permission_code 权限编码
 * @return bool 是否有权限
 */
function has_permission($permission_code)
{
    // 未登录用户没有任何权限
    if (!isset($_SESSION['user_id']) || !isset($_SESSION['role_id'])) {
        return false;
    }

    // 系统管理员拥有所有权限
    if ($_SESSION['role_id'] == 1) {
        return true;
    }

    // 获取用户角色权限
    $permissions = get_role_permissions($_SESSION['role_id']);
    if (empty($permissions)) {
        return false;
    }

    // 检查具体权限
    return in_array($permission_code, $permissions);
}

/**
 * 获取角色权限
 * @param int $role_id 角色ID
 * @return array 权限编码数组
 */
function get_role_permissions($role_id)
{
    global $pdo;

    try {
        $stmt = $pdo->prepare("
            SELECT p.code 
            FROM permissions p 
            INNER JOIN role_permissions rp ON p.id = rp.permission_id 
            WHERE rp.role_id = ?
        ");
        $stmt->execute([$role_id]);
        $results = $stmt->fetchAll(PDO::FETCH_COLUMN);

        return $results;
    } catch (PDOException $e) {
        error_log("获取角色权限失败: " . $e->getMessage());
        return [];
    }
}

/**
 * 检查当前用户是否有权限，如无权限则跳转到无权限页面
 * @param string $permission_code 权限编码
 */
function check_permission_or_die($permission_code)
{
    if (!has_permission($permission_code)) {
        // 记录未授权访问日志
        record_log("未授权访问: " . $permission_code);

        // 跳转到无权限页面
        header("Location: error.php?code=403");
        exit;
    }
}
