<?php
require_once '../database_config.php';

// 检查用户是否有权限访问此页面
check_permission_or_die('system_role');

// 记录访问日志
record_log('访问角色权限管理页面');

// 定义系统所有可用权限
$all_permissions = [
    'system' => [
        'view' => '系统设置-查看',
        'user' => '系统设置-用户管理',
        'role' => '系统设置-角色管理',
        'log' => '系统设置-日志查看',
        'backup' => '系统设置-数据备份',
        'language' => '系统设置-多语言设置'
    ],
    'booking' => [
        'view' => '预订管理-查看',
        'add' => '预订管理-添加',
        'edit' => '预订管理-编辑',
        'delete' => '预订管理-取消',
        'confirm' => '预订管理-确认',
        'channel' => '预订管理-渠道管理'
    ],
    'checkin' => [
        'view' => '入住管理-查看',
        'add' => '入住管理-办理入住',
        'edit' => '入住管理-修改入住信息',
        'deposit' => '入住管理-押金管理'
    ],
    'checkout' => [
        'view' => '退房管理-查看',
        'process' => '退房管理-办理退房',
        'bill' => '退房管理-账单结算',
        'invoice' => '退房管理-发票管理'
    ],
    'room' => [
        'view' => '客房管理-查看',
        'add' => '客房管理-添加房间',
        'edit' => '客房管理-编辑房间',
        'status' => '客房管理-房态管理',
        'type' => '客房管理-房型管理'
    ],
    'housekeeping' => [
        'view' => '客房服务-查看',
        'schedule' => '客房服务-排班安排',
        'task' => '客房服务-任务分配',
        'supplies' => '客房服务-物品管理',
    ],
    'maintenance' => [
        'view' => '维修管理-查看',
        'add' => '维修管理-报修登记',
        'process' => '维修管理-维修处理',
        'complete' => '维修管理-完成确认'
    ],
    'guest' => [
        'view' => '客户管理-查看',
        'add' => '客户管理-添加客户',
        'edit' => '客户管理-编辑客户',
        'vip' => '客户管理-会员管理',
        'points' => '客户管理-积分管理',
        'preference' => '客户管理-偏好管理'
    ],
    'finance' => [
        'view' => '财务管理-查看',
        'income' => '财务管理-收入管理',
        'expense' => '财务管理-支出管理',
        'report' => '财务管理-财务报表'
    ],
    'billing' => [
        'view' => '账单管理-查看',
        'add' => '账单管理-创建账单',
        'edit' => '账单管理-修改账单',
        'payment' => '账单管理-支付处理',
        'refund' => '账单管理-退款处理'
    ],
    'channel' => [
        'view' => 'OTA渠道-查看',
        'config' => 'OTA渠道-配置管理',
        'price' => 'OTA渠道-价格管理',
        'sync' => 'OTA渠道-数据同步'
    ],
    'inventory' => [
        'view' => '物资库存-查看',
        'purchase' => '物资库存-采购管理',
        'supplier' => '物资库存-供应商管理',
        'stock' => '物资库存-库存管理'
    ],
    'report' => [
        'occupancy' => '统计报表-入住率分析',
        'room' => '统计报表-房间利用率',
        'order' => '统计报表-订单分析',
        'finance' => '统计报表-财务分析'
    ],
    'marketing' => [
        'view' => '营销管理-查看',
        'sms' => '营销管理-短信营销',
        'email' => '营销管理-邮件营销',
        'coupon' => '营销管理-优惠券管理'
    ]
];

// 处理表单提交
if ($_SERVER['REQUEST_METHOD'] == 'POST' && isset($_POST['action'])) {
    // 添加角色
    if ($_POST['action'] == 'add') {
        $role_name = isset($_POST['role_name']) ? trim($_POST['role_name']) : '';
        $permissions = isset($_POST['permissions']) ? $_POST['permissions'] : [];

        if (empty($role_name)) {
            $error = '角色名称不能为空！';
        } else {
            try {
                // 检查角色名是否已存在
                $stmt = $pdo->prepare("SELECT COUNT(*) as count FROM sys_roles WHERE role_name = ?");
                $stmt->execute([$role_name]);
                $result = $stmt->fetch();

                if ($result['count'] > 0) {
                    $error = '角色名称已存在！';
                } else {
                    // 权限处理
                    $permissions_json = json_encode(process_permissions($permissions));

                    // 添加角色
                    $stmt = $pdo->prepare("INSERT INTO sys_roles (role_name, permissions) VALUES (?, ?)");
                    $result = $stmt->execute([$role_name, $permissions_json]);

                    if ($result) {
                        record_log("添加角色：{$role_name}");
                        $success = '角色添加成功！';
                    } else {
                        $error = '角色添加失败！';
                    }
                }
            } catch (PDOException $e) {
                $error = '系统错误：' . $e->getMessage();
            }
        }
    }
    // 编辑角色
    elseif ($_POST['action'] == 'edit') {
        $role_id = isset($_POST['role_id']) ? intval($_POST['role_id']) : 0;
        $role_name = isset($_POST['role_name']) ? trim($_POST['role_name']) : '';
        $permissions = isset($_POST['permissions']) ? $_POST['permissions'] : [];

        if ($role_id <= 0 || empty($role_name)) {
            $error = '请填写完整角色信息！';
        } else {
            try {
                // 检查角色名是否已被其他角色使用
                $stmt = $pdo->prepare("SELECT COUNT(*) as count FROM sys_roles WHERE role_name = ? AND id != ?");
                $stmt->execute([$role_name, $role_id]);
                $result = $stmt->fetch();

                if ($result['count'] > 0) {
                    $error = '角色名称已被其他角色使用！';
                } else {
                    // 权限处理
                    $permissions_json = json_encode(process_permissions($permissions));

                    // 更新角色
                    $stmt = $pdo->prepare("UPDATE sys_roles SET role_name = ?, permissions = ? WHERE id = ?");
                    $result = $stmt->execute([$role_name, $permissions_json, $role_id]);

                    if ($result) {
                        record_log("编辑角色：{$role_name}");
                        $success = '角色更新成功！';
                    } else {
                        $error = '角色更新失败！';
                    }
                }
            } catch (PDOException $e) {
                $error = '系统错误：' . $e->getMessage();
            }
        }
    }
    // 删除角色
    elseif ($_POST['action'] == 'delete') {
        $role_id = isset($_POST['role_id']) ? intval($_POST['role_id']) : 0;

        if ($role_id <= 0) {
            $error = '无效的角色ID！';
        } elseif ($role_id == 1) {
            $error = '系统管理员角色不能删除！';
        } else {
            try {
                // 检查是否有用户使用此角色
                $stmt = $pdo->prepare("SELECT COUNT(*) as count FROM sys_users WHERE role_id = ?");
                $stmt->execute([$role_id]);
                $result = $stmt->fetch();

                if ($result['count'] > 0) {
                    $error = '此角色下有用户，不能删除！';
                } else {
                    // 获取角色名，用于日志记录
                    $stmt = $pdo->prepare("SELECT role_name FROM sys_roles WHERE id = ?");
                    $stmt->execute([$role_id]);
                    $role = $stmt->fetch();

                    if (!$role) {
                        $error = '角色不存在！';
                    } else {
                        // 删除角色
                        $stmt = $pdo->prepare("DELETE FROM sys_roles WHERE id = ?");
                        $result = $stmt->execute([$role_id]);

                        if ($result) {
                            record_log("删除角色：{$role['role_name']}");
                            $success = '角色删除成功！';
                        } else {
                            $error = '角色删除失败！';
                        }
                    }
                }
            } catch (PDOException $e) {
                $error = '系统错误：' . $e->getMessage();
            }
        }
    }
}

// 处理权限数据结构
function process_permissions($permissions)
{
    $result = [];

    // 如果有all权限，直接返回
    if (isset($permissions['all'])) {
        $result['all'] = true;
        return $result;
    }

    // 处理一级权限
    foreach ($permissions as $key => $value) {
        // 键名中包含点号的是子权限
        if (strpos($key, '.') !== false) {
            list($parent, $child) = explode('.', $key);

            // 初始化父权限数组（如果还没有）
            if (!isset($result[$parent]) || !is_array($result[$parent])) {
                $result[$parent] = [];
            }

            // 设置子权限
            $result[$parent][$child] = true;
        } else {
            // 设置父权限
            $result[$key] = true;
        }
    }

    return $result;
}

// 获取所有角色
try {
    $stmt = $pdo->query("SELECT * FROM sys_roles ORDER BY id");
    $roles = $stmt->fetchAll();
} catch (PDOException $e) {
    $error = '获取角色列表失败：' . $e->getMessage();
    $roles = [];
}

// 权限列表
$permission_list = [
    'system' => '系统设置',
    'system.view' => '查看系统设置',
    'system.user' => '用户管理',
    'system.role' => '角色管理',
    'system.log' => '日志管理',
    'system.backup' => '数据备份',
    'system.language' => '多语言设置',

    'booking' => '预订管理',
    'booking.view' => '查看预订',
    'booking.add' => '添加预订',
    'booking.edit' => '编辑预订',
    'booking.delete' => '取消预订',
    'booking.confirm' => '确认预订',
    'booking.channel' => '渠道管理',

    'checkin' => '入住管理',
    'checkin.view' => '查看入住',
    'checkin.add' => '办理入住',
    'checkin.edit' => '修改入住信息',
    'checkin.deposit' => '押金管理',

    'checkout' => '退房管理',
    'checkout.view' => '查看退房',
    'checkout.process' => '办理退房',
    'checkout.bill' => '账单结算',
    'checkout.invoice' => '发票管理',

    'room' => '客房管理',
    'room.view' => '查看房间',
    'room.add' => '添加房间',
    'room.edit' => '编辑房间',
    'room.status' => '房态管理',
    'room.type' => '房型管理',

    'housekeeping' => '客房服务',
    'housekeeping.view' => '查看服务',
    'housekeeping.schedule' => '排班安排',
    'housekeeping.task' => '任务分配',
    'housekeeping.supplies' => '物品管理',

    'maintenance' => '维修管理',
    'maintenance.view' => '查看维修',
    'maintenance.add' => '报修登记',
    'maintenance.process' => '维修处理',
    'maintenance.complete' => '完成确认',

    'guest' => '客户管理',
    'guest.view' => '查看客户',
    'guest.add' => '添加客户',
    'guest.edit' => '编辑客户',
    'guest.vip' => '会员管理',
    'guest.points' => '积分管理',
    'guest.preference' => '偏好管理',

    'finance' => '财务管理',
    'finance.view' => '查看财务',
    'finance.income' => '收入管理',
    'finance.expense' => '支出管理',
    'finance.report' => '财务报表',

    'billing' => '账单管理',
    'billing.view' => '查看账单',
    'billing.add' => '创建账单',
    'billing.edit' => '修改账单',
    'billing.payment' => '支付处理',
    'billing.refund' => '退款处理',

    'channel' => 'OTA渠道管理',
    'channel.view' => '查看渠道',
    'channel.config' => '配置管理',
    'channel.price' => '价格管理',
    'channel.sync' => '数据同步',

    'inventory' => '物资库存',
    'inventory.view' => '查看库存',
    'inventory.purchase' => '采购管理',
    'inventory.supplier' => '供应商管理',
    'inventory.stock' => '库存管理',

    'report' => '统计报表',
    'report.occupancy' => '入住率分析',
    'report.room' => '房间利用率',
    'report.order' => '订单分析',
    'report.finance' => '财务分析',

    'marketing' => '营销管理',
    'marketing.view' => '查看营销',
    'marketing.sms' => '短信营销',
    'marketing.email' => '邮件营销',
    'marketing.coupon' => '优惠券管理',

    'all' => '所有权限'
];

// 分组权限，便于前端展示
$permission_groups = [
    '系统设置' => ['system', 'system.view', 'system.user', 'system.role', 'system.log', 'system.backup', 'system.language'],
    '预订管理' => ['booking', 'booking.view', 'booking.add', 'booking.edit', 'booking.delete', 'booking.confirm', 'booking.channel'],
    '入住与退房' => ['checkin', 'checkin.view', 'checkin.add', 'checkin.edit', 'checkin.deposit', 'checkout', 'checkout.view', 'checkout.process', 'checkout.bill', 'checkout.invoice'],
    '客房管理' => ['room', 'room.view', 'room.add', 'room.edit', 'room.status', 'room.type', 'housekeeping', 'housekeeping.view', 'housekeeping.schedule', 'housekeeping.task', 'housekeeping.supplies'],
    '维修管理' => ['maintenance', 'maintenance.view', 'maintenance.add', 'maintenance.process', 'maintenance.complete'],
    '客户管理' => ['guest', 'guest.view', 'guest.add', 'guest.edit', 'guest.vip', 'guest.points', 'guest.preference'],
    '财务管理' => ['finance', 'finance.view', 'finance.income', 'finance.expense', 'finance.report', 'billing', 'billing.view', 'billing.add', 'billing.edit', 'billing.payment', 'billing.refund'],
    'OTA渠道管理' => ['channel', 'channel.view', 'channel.config', 'channel.price', 'channel.sync'],
    '物资与库存' => ['inventory', 'inventory.view', 'inventory.purchase', 'inventory.supplier', 'inventory.stock'],
    '统计与报表' => ['report', 'report.occupancy', 'report.room', 'report.order', 'report.finance'],
    '营销管理' => ['marketing', 'marketing.view', 'marketing.sms', 'marketing.email', 'marketing.coupon'],
    '特殊权限' => ['all']
];
?>

<!DOCTYPE html>
<html lang="zh-CN">

<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title><?php echo title; ?></title>
    <link rel="stylesheet" href="../style.css">
</head>

<body>
    <div class="container">
        <div class="header-local">
            <h2>角色权限管理</h2>
            <div class="breadcrumb">
                <a href="index.php">系统设置</a> &gt; 角色权限管理
            </div>
        </div>

        <div class="content">
            <?php if (isset($error)): ?>
                <div class="alert alert-danger"><?php echo $error; ?></div>
            <?php endif; ?>

            <?php if (isset($success)): ?>
                <div class="alert alert-success"><?php echo $success; ?></div>
            <?php endif; ?>

            <div class="action-bar">
                <button class="btn btn-primary" onclick="showAddForm()">添加角色</button>
            </div>

            <!-- 添加角色表单 -->
            <div id="addRoleForm" class="form-container" style="display: none;">
                <h3>添加角色</h3>
                <form method="post" action="">
                    <input type="hidden" name="action" value="add">

                    <div class="form-group">
                        <label for="role_name">角色名称：</label>
                        <input type="text" id="role_name" name="role_name" required>
                    </div>

                    <div class="form-group">
                        <label>权限设置：</label>
                        <div class="options-container">
                            <?php foreach ($permission_groups as $group => $permissions): ?>
                                <div class="option-group">
                                    <h4 class="group-header"><?php echo $group; ?></h4>
                                    <?php foreach ($permissions as $permission): ?>
                                        <div class="checkbox-item">
                                            <label>
                                                <input type="checkbox" name="permissions[<?php echo $permission; ?>]" value="1" <?php echo $permission == 'all' ? 'onclick="toggleAllPermissions(this)"' : ''; ?>>
                                                <?php echo $permission_list[$permission]; ?>
                                            </label>
                                        </div>
                                    <?php endforeach; ?>
                                </div>
                            <?php endforeach; ?>
                        </div>
                    </div>

                    <div class="form-actions">
                        <button type="submit" class="btn btn-success">保存</button>
                        <button type="button" class="btn btn-secondary" onclick="hideAddForm()">取消</button>
                    </div>
                </form>
            </div>

            <!-- 编辑角色表单 -->
            <div id="editRoleForm" class="form-container" style="display: none;">
                <h3>编辑角色</h3>
                <form method="post" action="">
                    <input type="hidden" name="action" value="edit">
                    <input type="hidden" name="role_id" id="edit_role_id">

                    <div class="form-group">
                        <label for="edit_role_name">角色名称：</label>
                        <input type="text" id="edit_role_name" name="role_name" required>
                    </div>

                    <div class="form-group">
                        <label>权限设置：</label>
                        <div class="options-container" id="edit_permissions_container">
                            <?php foreach ($permission_groups as $group => $permissions): ?>
                                <div class="option-group">
                                    <h4 class="group-header"><?php echo $group; ?></h4>
                                    <?php foreach ($permissions as $permission): ?>
                                        <div class="checkbox-item">
                                            <label>
                                                <input type="checkbox" name="permissions[<?php echo $permission; ?>]" value="1" id="edit_perm_<?php echo str_replace('.', '_', $permission); ?>" <?php echo $permission == 'all' ? 'onclick="toggleAllPermissionsEdit(this)"' : ''; ?>>
                                                <?php echo $permission_list[$permission]; ?>
                                            </label>
                                        </div>
                                    <?php endforeach; ?>
                                </div>
                            <?php endforeach; ?>
                        </div>
                    </div>

                    <div class="form-actions">
                        <button type="submit" class="btn btn-success">保存</button>
                        <button type="button" class="btn btn-secondary" onclick="hideEditForm()">取消</button>
                    </div>
                </form>
            </div>

            <!-- 角色列表 -->
            <div class="table-container">
                <table class="data-table">
                    <thead>
                        <tr>
                            <th>ID</th>
                            <th>角色名称</th>
                            <th>权限数量</th>
                            <th>创建时间</th>
                            <th>操作</th>
                        </tr>
                    </thead>
                    <tbody>
                        <?php if (empty($roles)): ?>
                            <tr>
                                <td colspan="5" class="text-center">暂无角色数据</td>
                            </tr>
                        <?php else: ?>
                            <?php foreach ($roles as $role): ?>
                                <?php
                                $permissions = !empty($role['permissions']) ? json_decode($role['permissions'], true) : [];
                                $permissionCount = 0;

                                if (!empty($permissions)) {
                                    if (isset($permissions['all']) && $permissions['all'] === true) {
                                        $permissionCount = '全部权限';
                                    } else {
                                        // 计算权限数量
                                        foreach ($permissions as $key => $value) {
                                            if (is_array($value)) {
                                                $permissionCount += count($value);
                                            } else {
                                                $permissionCount++;
                                            }
                                        }
                                    }
                                }
                                ?>
                                <tr>
                                    <td><?php echo $role['id']; ?></td>
                                    <td><?php echo htmlspecialchars($role['role_name']); ?></td>
                                    <td><?php echo $permissionCount; ?></td>
                                    <td><?php echo $role['create_time']; ?></td>
                                    <td>
                                        <button class="btn btn-sm btn-primary" onclick="editRole(<?php echo $role['id']; ?>, '<?php echo htmlspecialchars($role['role_name']); ?>', <?php echo htmlspecialchars(json_encode($permissions)); ?>)">编辑</button>
                                        <?php if ($role['id'] != 1): // 防止删除超级管理员角色 
                                        ?>
                                            <button class="btn btn-sm btn-danger" onclick="deleteRole(<?php echo $role['id']; ?>, '<?php echo htmlspecialchars($role['role_name']); ?>')">删除</button>
                                        <?php endif; ?>
                                    </td>
                                </tr>
                            <?php endforeach; ?>
                        <?php endif; ?>
                    </tbody>
                </table>
            </div>

            <!-- 删除角色表单 -->
            <form id="deleteRoleForm" method="post" action="" style="display: none;">
                <input type="hidden" name="action" value="delete">
                <input type="hidden" name="role_id" id="delete_role_id">
            </form>
        </div>
    </div>

    <script>
        // 显示添加角色表单
        function showAddForm() {
            document.getElementById('addRoleForm').style.display = 'block';
        }

        // 隐藏添加角色表单
        function hideAddForm() {
            document.getElementById('addRoleForm').style.display = 'none';
        }

        // 显示编辑角色表单
        function editRole(id, roleName, permissions) {
            document.getElementById('edit_role_id').value = id;
            document.getElementById('edit_role_name').value = roleName;

            // 重置所有权限
            var checkboxes = document.querySelectorAll('#edit_permissions_container input[type="checkbox"]');
            checkboxes.forEach(function(checkbox) {
                checkbox.checked = false;
            });

            // 设置已有权限
            if (permissions) {
                // 处理all权限
                if (permissions.all) {
                    document.getElementById('edit_perm_all').checked = true;
                    // 选中所有权限
                    checkboxes.forEach(function(checkbox) {
                        checkbox.checked = true;
                    });
                } else {
                    // 处理一级权限
                    for (var key in permissions) {
                        if (permissions.hasOwnProperty(key)) {
                            var value = permissions[key];

                            if (typeof value === 'boolean' && value) {
                                var checkbox = document.getElementById('edit_perm_' + key);
                                if (checkbox) {
                                    checkbox.checked = true;
                                }
                            } else if (typeof value === 'object') {
                                // 处理二级权限
                                for (var subKey in value) {
                                    if (value.hasOwnProperty(subKey) && value[subKey]) {
                                        var checkbox = document.getElementById('edit_perm_' + key + '_' + subKey);
                                        if (checkbox) {
                                            checkbox.checked = true;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            document.getElementById('editRoleForm').style.display = 'block';
        }

        // 隐藏编辑角色表单
        function hideEditForm() {
            document.getElementById('editRoleForm').style.display = 'none';
        }

        // 删除角色确认
        function deleteRole(id, roleName) {
            if (confirm('确定要删除角色 "' + roleName + '" 吗？')) {
                document.getElementById('delete_role_id').value = id;
                document.getElementById('deleteRoleForm').submit();
            }
        }

        // 选中"所有权限"时，自动选中所有权限
        function toggleAllPermissions(checkbox) {
            var checkboxes = document.querySelectorAll('#addRoleForm input[type="checkbox"]');
            checkboxes.forEach(function(item) {
                item.checked = checkbox.checked;
            });
        }

        // 编辑表单中的全选功能
        function toggleAllPermissionsEdit(checkbox) {
            var checkboxes = document.querySelectorAll('#editRoleForm input[type="checkbox"]');
            checkboxes.forEach(function(item) {
                item.checked = checkbox.checked;
            });
        }
    </script>
</body>

</html>